<!--
 Deployment page.
 update.php
 Lists, adds and removes PageBoxe archives.
 Version 0.01
 Copyright (c) 2001-2002 Alexis Grandemange
 Mail: alexis.grandemange@pagebox.net
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; version
 2.1 of the License.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU Lesser General Public License for more details.
 A copy of the GNU Lesser General Public License lesser.txt should be
 included in the distribution.
 -->
<html>
<head>
<meta http-equiv="Cache-Control" content="no-cache">
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
<meta name="Description" content="PHP Pagebox update">
<meta name="keywords" content="Update, PHP, Pagebox">
<link rel="stylesheet" href="pagebox.css" type="text/css">
<script src="pagebox.js"></script>
<title>PHP Pagebox update</title>
</head>
<body>
<?php
/**
 * Describes a PageBox archive.
 */
class Archive {
  /** Download location (repository) */
  var $download;
  /** Archive owner (publisher) */
  var $owner;
  /** Archive size */
  var $size;
  /** Archive last modified time */
  var $date;
  /** Archive documentation URL */
  var $doc;
  /**
   * Constructor.
   * Parameters:
   * l: Download location
   * o: Archive owner
   * s: Archive size
   * d: Archive last modified time
   * u: Archive documentation URL
   */
  function Archive($l, $o, $s, $d, $u) {
    $this->download = $l;
    $this->owner = $o;
    $this->size = $s;
    $this->date = $d;
    $this->doc = $u;
  }
}
/**
 * Describes deployed archives.
 * Serialized in pbArchives.pb.
 */
class PbArchives {
  /**
   * Archive array.
   * Key: archive name
   * Value: Archive object
   */
  var $archives;
  /**
   * Command used to inflate the archive.
   * 1 parameter: archive name.
   * Read from inflate.pb.
   */
  var $inflate;
  /**
   * Command used to remove an inflated archive.
   * 1 parameter: archive name.
   * Read from delete.pb.
   */
  var $delete;
  /**
   * Constructor.
   * Creates inflate.
   */
  function PbArchives() {
    if ($this->inflate != NULL) {
      return;
    }
    if (!file_exists("inflate.pb")) {
      return;
    }
    $fcmd = fopen("inflate.pb", "r");
    while ($iar = fgetcsv ($fcmd, 255, ":")) {
      if (($iar == NULL) || (count($iar) < 2)) {
        continue;
      }
      $this->inflate[trim($iar[0])] = trim($iar[1]);
    }
    fclose($fcmd);
    if ($this->delete != NULL) {
      return;
    }
    if (!file_exists("delete.pb")) {
      return;
    }
    $fcmd = fopen("delete.pb", "r");
    while ($iar = fgetcsv ($fcmd, 255, ":")) {
      if (($iar == NULL) || (count($iar) < 2)) {
        continue;
      }
      $this->delete[trim($iar[0])] = trim($iar[1]);
    }
    fclose($fcmd);
  }
  /**
   * Adds an archive.
   * Returns a status (String)
   * Parameters:
   * name: archive name
   * download: Download location
   * owner: Archive owner
   * size: Archive size
   * date: Archive last modified time
   * doc: Archive documentation URL
   */
  function add($name, $download, $owner, $size, $date, $doc) {
    if ($this->inflate == NULL) {
      return "Unsupported extension: inflate.pb not found";
    }
    $pieces = explode(".", $name);
    $inflate = $this->inflate[$pieces[1]];
    if ($inflate == NULL) {
      return "Unsupported extension: " . $pieces[1];
    }
    if ($this->archives[$name] != NULL) {
      $a = $this->archives[$name];
      if ($a->owner != $owner) {
        return "Invalid owner";
      } else {
        if (file_exists($name)) {
          unlink($name);
        }
        unset($this->archives[$name]);
      }
    }
    // Download the archive
    $farch = fopen('http://' . $download . "/" . $name, "br");
    if ($farch == FALSE) {
      return "http://" . $download . "/" . $name . " not found";
    }
    $rd = fread ($farch, $size);
    fclose($farch);
    $farch = fopen($name, "wb");
    if ($farch == FALSE) {
      return "Unable to write " . $name;
    }
    fwrite($farch, $rd, $size);
    fclose($farch);
    // Inflate the archive
    if ($inflate != "-") {
      exec($inflate . " " . $pieces[0]);
    }
    $status = $name . " installed";
    $a = new Archive($download, $owner, $size, $date, $doc);
    $this->archives[$name] = $a;
    return $status;
  }
  /**
   * Deletes an archive.
   * Returns a status (string).
   * Parameter:
   * name: archive name.
   */
  function delete($name) {
    if ($this->archives[$name] == NULL) {
      return $name . " is not installed";
    }
    unset($this->archives[$name]);
    // Inflate the archive
    if ($this->delete != NULL) {
      $pieces = explode(".", $name);
      $delete = $this->delete[$pieces[1]];
      if ($delete != NULL) {
        exec($delete . " " . $pieces[0]);
      }
    }
    unlink($name);
    return $name . " uninstalled";
  }
}
/**
 * Restores pbArchive.
 */
$status = "";
if (file_exists("pbArchives.pb")) {
  $s = implode("", @file("pbArchives.pb"));
  $PbArchivesObj = unserialize($s);
} else {
  $PbArchivesObj = new PbArchives;
}
/**
 * Request analysis
 * Expects:
 * update.php?download=...&from=...&owner=...&size=...&date=... or
 * update.php?delete=...
 */
$isModified = false;
if ($HTTP_GET_VARS["download"] != NULL) {
  $status = $PbArchivesObj->add($HTTP_GET_VARS["download"], $HTTP_GET_VARS["from"], $HTTP_GET_VARS["owner"],
    $HTTP_GET_VARS["size"], $HTTP_GET_VARS["date"], $HTTP_GET_VARS["doc"]);
  $isModified = true;
} elseif ($HTTP_GET_VARS["delete"] != NULL) {
  $status = $PbArchivesObj->delete($HTTP_GET_VARS["delete"]);
  $isModified = true;
}
/**
 * Save pbArchive.
 */
if ($isModified) {
  if ($PbArchivesObj->archives != NULL) {
    reset($PbArchivesObj->archives);
    ksort($PbArchivesObj->archives, SORT_STRING);
  }
  $s = serialize($PbArchivesObj);
  $fp = fopen("pbArchives.pb", "w");
  fputs($fp, $s);
  fclose($fp);
}
?>
<table width="100%"><tr>
<td align="left" width="8%">
<a href="http://pagebox.net/php/php-index.html" style="text-decoration:none;background:white;color:white">
<img src="logo.gif" alt="PHP Pagebox update" border="0"></a></td>
<td align="left"><h1>Update</h1></td>
<td align="right"><a href="environment.php" alt="PHP environment">Environment</a></td>
<td align="right"><a href="mailto:support@pagebox.net">
<img src="mail.gif" alt="support@pagebox.net" border="0"></a></td></tr></table>
<table width="100%"><tr bgcolor="maroon">
<th width="16%">Archive</th><th width="24%">Download location</th>
<th width="8%">Owner</th><th width="20%">Date</th><th width="8%">Size</th>
<th width="24%">Documentation</th></tr>
<?php
/**
 * Lists deployed archives.
 */
if ($PbArchivesObj->archives != NULL) {
  while (list($key, $value) = each ($PbArchivesObj->archives)) {
    echo '<td width="16%">', $key, '</td>';
    echo '<td width="24%">', $value->download, '</td>';
    echo '<td width="8%">', $value->owner, '</td>';
    echo '<td width="20%">', $value->date, '</td>';
    echo '<td width="8%">', $value->size, '</td>';
    echo '<td width="24%"><a href="', $value->doc, '">', $value->doc, '</a></td></tr>';
  }
}
?>
</table>
<p class="timestamp"><?= $status ?></p>
<p align="center">&#169;2002 Alexis Grandemange.
<font class="timestamp">Last modified
<script>document.write(document.lastModified);</script>.
</font></p>
</body>
</html>