<?php
/**
 * Repository of PHP PageBox
 * archive.inc
 * Version 0.0.3
 * Changes 0.0.2:
 * - lock/unlock
 * - audit
 * - cleanup
 * Changes 0.0.3:
 * - archive subscriber
 * - documentation URL
 * Copyright (c) 2001-2002 Alexis Grandemange
 * Mail: alexis.grandemange@pagebox.net
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version
 * 2.1 of the License.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 * A copy of the GNU Lesser General Public License lesser.txt should be
 * included in the distribution.
 */
/**
 * Describes archives managed by a Repository.
 * Serialized in archives.rep.
 */
// Archive repository
class ArchiveRep {
  /**
   * Array of archives.
   * Key: archive name.
   * Value: archive publisher (owner).
   */ 
  var $archives;
  /**
   * Array of archive documentation URLs.
   * Key: archive name.
   * Value: archive documentation URL.
   */ 
  var $docs;
  /**
   * This URL.
   */
  var $download;
  /**
   * Constructor.
   * Set download URL.
   */
  function ArchiveRep($d) {
    $this->download = $d;
  }
  /**
   * Adds an archive to the repository.
   * Returns a status string.
   * Parameters:
   * name: archive name
   * owner: archive owner.
   * docURL: documentation URL.
   */
  function add($name, $owner, $docURL) {
    // Before PHP 4.0.6
    // if (array_key_exists($name, $this->archives)) {
    // PHP 4.0.6
    $RepoSubsObj = restore_subscribers($this->download);
    if (($this->archives != NULL) && key_exists($name, $this->archives)) {
      if ($this->archives[$name] != $owner) {
        return "The archive already exists and doesn't belongs to you";
      } else {
        $status = "'" . $name . "' updated";
        // We need to update archive subscribers only here
        $toSave = false;
        if ($RepoSubsObj->asubscribers != NULL) {
          while (list($key, $value) = each ($RepoSubsObj->asubscribers)) {
            if (($value->pbState != "pending unsubscribe") && ($value->arch[$name] != NULL)) {
              $d = deploy($key, $name, $owner, $docURL, $this->download);
              if ($d != FALSE) {
                $RepoSubsObj->asubscribers[$key]->arch[$name] = $d;
                $toSave = true;
              }	// if
            }	// if
          }	// while
        }	// if
        if ($toSave) {
          save_subscribers($RepoSubsObj);
        }
      }		// update
    } else {
      $status = "'" . $name . "' created";
      $this->archives[$name] = $owner;
    }
    $this->docs[$name] = $docURL;
    if ($RepoSubsObj->subscribers != NULL) {
      while (list($key, $value) = each ($RepoSubsObj->subscribers)) {
        if ($value->pbState != "pending unsubscribe") {
          $d = deploy($key, $name, $owner, $docURL, $this->download);
          if ($d != FALSE) {
            $RepoSubsObj->subscribers[$key]->arch[$name] = $d;
          }
        }
      }
      save_subscribers($RepoSubsObj);
    }
    return $status;
  }
  /**
   * Remove an archive from the repository.
   * Returns a status string.
   * Parameter:
   * name: archive name.
   * isForce: If true remove the archive from the subscriber even if the subscriber cannot be contacted
   */
  function delete($name, $isForce) {
    // Before PHP 4.0.6
    // if (!array_key_exists($name, $this->archives)) {
    // PHP 4.0.6
    if (($this->archives == NULL) || !key_exists($name, $this->archives)) {
      return "'" . $name . "' doesn't exist";
    }
    $RepoSubsObj = restore_subscribers($this->download);
    if ($RepoSubsObj->subscribers != NULL) {
      while (list($key, $value) = each ($RepoSubsObj->subscribers)) {
        $rc = true;
        if ($value->arch[$name] != "pending deploy") {
          $rc = undeploy($key, $name);
        }
        if ($rc || $isForce) {
          unset($RepoSubsObj->subscribers[$key]->arch[$name]);
          if (($RepoSubsObj->subscribers[$key]->pbState == "pending unsubscribe") &&
            ($RepoSubsObj->subscribers[$key]->arch == NULL)) {
            unset($RepoSubsObj->subscribers[$key]);
          }
        } else {
          $RepoSubsObj->subscribers[$key]->arch[$name] = "pending undeploy";
        }
      }
      save_subscribers($RepoSubsObj);
    }
    if ($RepoSubsObj->asubscribers != NULL) {
      while (list($key, $value) = each ($RepoSubsObj->asubscribers)) {
        $rc = true;
        if (($value->arch[$name] != NULL) && ($value->arch[$name] != "pending deploy")) {
          $rc = undeploy($key, $name);
        }
        if ($rc || $isForce) {
          unset($RepoSubsObj->asubscribers[$key]->arch[$name]);
          if (($RepoSubsObj->asubscribers[$key]->pbState == "pending unsubscribe") &&
            ($RepoSubsObj->asubscribers[$key]->arch == NULL)) {
            unset($RepoSubsObj->asubscribers[$key]);
          }
        } else {
          $RepoSubsObj->asubscribers[$key]->arch[$name] = "pending undeploy";
        }
      }
      save_subscribers($RepoSubsObj);
    }
    if (file_exists("../download/" . $name)) {
      unlink("../download/" . $name);
    }
    unset($this->archives[$name]);
    return $name . " deleted";
  }
}
/**
 * Restores ArchiveRep from archive.rep.
 * Creates ArchiveRep if it doesn't exist.
 * Returns ArchiveRep.
 * Parameter: download URL.
 */
function restore_archives($download) {
  $ar = NULL;
  if (file_exists("../repository/archives.rep")) {
    $s = implode("", @file("../repository/archives.rep"));
    $ar = unserialize($s);
    $ar->download = $download;
  }
  if ($ar == NULL) {
    $ar = new ArchiveRep($download);
  }
  return $ar;
}
/**
 * Serialize ArchiveRep into archive.rep if it is not empty.
 * Parameter:
 * ar: ArchiveRep.
 */
function save_archives($ar) {
  $ar->download = $download;
  if ($ar->archives != NULL) {
    reset($ar->archives);
    ksort($ar->archives, SORT_STRING);
  }
  $s = serialize($ar);
  $fp = fopen("../repository/archives.rep", "w");
  fputs($fp, $s);
  fclose($fp);
}
/**
 * Describes a subscriber.
 */
class Subscriber {
  /** Subscriber state (string). */
  var $pbState;
  /**
   * Subscriber array of archives.
   * Key: archive name.
   * Value: archive status (string).
   */
  var $arch;
  /** Constructor. */
  function Subscriber($pbs = "", $a = "") {
    $this->pbState = $pbs;
    $this->arch = $a;
  }
}
/**
 * Describes Repository subscribers.
 * Serialized in subscribers.rep.
 */
class RepoSubs {
  /**
   * Subscriber array.
   * Key: subscriber URL (http://...)
   * Value: Subscriber object.
   */
  var $subscribers;
  /**
   * Archive subscriber array.
   * Key: subscriber URL (http://...)
   * Value: Subscriber object.
   */
  var $asubscribers;
  /**
   * This URL.
   */
  var $download;
  /**
   * Archive repository (ArchiveRep class).
   */
  var $archRep;
  /**
   * Initialize.
   * Parameter: download URL.
   */
  function initialize($d) {
    $this->download = $d;
    $this->archRep = restore_archives($this->download);
  }
  /**
   * Terminate. Cleanup $archRep and $download.
   */
  function terminate() {
    $this->download = NULL;
    $this->archRep = NULL;
  }
  /**
   * Adds a subscriber.
   * Returns a subscription status.
   * Parameter:
   * loc: URL of the subscriber.
   */
  function subscribe($loc) {
    if ($this->subscribers[$loc] != NULL) {
      return $loc . " already subscribed";
    }
    if ($this->archRep->archives != NULL) {
      while (list($key, $value) = each ($this->archRep->archives)) {
        $d = deploy($loc, $key, $value, $this->archRep->docs[$key], $this->download);
        if ($d != FALSE) {
          $as[$key] = $d;
        }
      }
    }
    $s = new Subscriber("active", $as);
    $this->subscribers[$loc] = $s;
    return $loc . " subscribed";
  }
  /**
   * Adds an archive subscriber.
   * Returns a subscription status.
   * Parameter:
   * loc: URL of the subscriber.
   */
  function asubscribe($loc) {
    if ($this->asubscribers[$loc] != NULL) {
      return $loc . " already subscribed";
    }
    $this->asubscribers[$loc] = new Subscriber("active", NULL);
    return $loc . " subscribed";
  }
  /**
   * Remove a subscriber.
   * Returns an unsubscribe status.
   * Parameter:
   * loc: URL of the subscriber.
   * isForce: If true force subscriber removal even if it cannot be contacted
   */
  function unsubscribe($loc, $isForce) {
    if ($this->subscribers[$loc] == NULL) {
      return $loc . " is not subscribed";
    }
    $s = $this->subscribers[$loc];
    $nbErr = 0;
    if ($s->arch != NULL) {
      while (list($key, $value) = each ($s->arch)) {
        if ($value != "pending deploy") {
          if (undeploy($loc, $key)) {
            unset($this->subscribers[$loc]->arch[$key]);
          } else {
            $this->subscribers[$loc]->arch[$key] = "pending undeploy";
            $nbErr = $nbErr + 1;
          }
        } else {
          unset($this->subscribers[$loc]->arch[$key]);
        }
      }
    }
    if (($nbErr > 0) && !$isForce) {
      $this->subscribers[$loc]->pbState = "pending unsubscribe";
      return $name . " pending unsubscribe";
    }
    unset($this->subscribers[$loc]);
    return $loc . " unsubscribed";
  }
  /**
   * Remove an archive subscriber.
   * Returns an unsubscribe status.
   * Parameters:
   * loc: URL of the subscriber.
   * isForce: If true force subscriber removal even if it cannot be contacted
   */
  function unasubscribe($loc, $isForce) {
    if ($this->asubscribers[$loc] == NULL) {
      return $loc . " is not subscribed";
    }
    $s = $this->asubscribers[$loc];
    $nbErr = 0;
    if ($s->arch != NULL) {
      while (list($key, $value) = each ($s->arch)) {
        if ($value != "pending deploy") {
          if (undeploy($loc, $key)) {
            unset($this->asubscribers[$loc]->arch[$key]);
          } else {
            $this->asubscribers[$loc]->arch[$key] = "pending undeploy";
            $nbErr = $nbErr + 1;
          }
        } else {
          unset($this->asubscribers[$loc]->arch[$key]);
        }
      }
    }
    if (($nbErr > 0) && !$isForce) {
      $this->asubscribers[$loc]->pbState = "pending unsubscribe";
      return $name . " pending unsubscribe";
    }
    unset($this->asubscribers[$loc]);
    return $loc . " unsubscribed";
  }
  /**
   * Adds an archive to an archive subscriber.
   * Parameters:
   * sub: subscriber URL.
   * arch: archive.
   */
  function add($sub, $arch) {
    $owner = NULL;
    if (($this->archRep->archives == NULL) ||
      (($owner = $this->archRep->archives[$arch]) == NULL)) {
      return;
    }
    $doc = $this->archRep->docs[$arch];
    $d = deploy($sub, $arch, $owner, $doc, $this->download);
    if ($d != FALSE) {
      $this->asubscribers[$sub]->arch[$arch] = $d;
    }
  }
  /**
   * Removes an archive from an archive subscriber.
   * Parameters:
   * sub: subscriber.
   * arch: archive.
   */
  function delete($sub, $arch) {
    if ($this->asubscribers[$sub]->arch[$arch] != "pending deploy") {
      if (undeploy($sub, $arch)) {
        unset($this->asubscribers[$sub]->arch[$arch]);
      } else {
        $this->asubscribers[$sub]->arch[$arch] = "pending undeploy";
      }
    } else {
      unset($this->asubscribers[$loc]->arch[$key]);
    }
  }
}
/**
 * Restores RepoSubs from subscribers.rep.
 * Creates RepoSubs if it doesn't exist.
 * Returns RepoSubs.
 * Parameter: download URL.
 */
function restore_subscribers($d) {
  $rso = NULL;
  if (file_exists("../repository/subscribers.rep")) {
    $s = implode("", @file("../repository/subscribers.rep"));
    $rso = unserialize($s);
    $rso->initialize($d);
  }
  if ($rso == NULL) {
    $rso = new RepoSubs();
    $rso->initialize($d);
  }
  return $rso;
}
/**
 * Serialize RepoSubs into subscribers.rep if it is not empty.
 * Parameter:
 * rso: RepoSubs.
 */
function save_subscribers($rso) {
  $rso->terminate();
  if ($rso->subscribers != NULL) {
    reset($rso->subscribers);
    ksort($rso->subscribers, SORT_STRING);
  }
  $s = serialize($rso);
  $fp = fopen("../repository/subscribers.rep", "w");
  fputs($fp, $s);
  fclose($fp);
}
/**
 * Deploys an archive on a PageBox.
 * Download location = archive.inc directory.
 * Makes a request update.php?download=...&from=...&owner=...&size=...&date=... or
 * Returns "active": deployed or "pending deploy": PageBox doesn't answer.
 */
function deploy($loc, $arch, $owner, $doc, $download) {
  $query = $loc . '?download=' . $arch . '&from=' . $download . '&owner=' . $owner .
    '&doc=' . $doc . '&size=' . filesize("../download/" . $arch) . '&date=' .
    rawurlencode(date("d M Y h:i:s", filemtime("../download/" . $arch)));
  // echo $query, "<br>";
  $fdeploy = fopen($query, "r");
  if ($fdeploy == FALSE) {
    return "pending deploy";
  }
  while (!feof ($fdeploy)) {
    $status = fgets($fdeploy, 255);
    // echo $status;
    if ((strstr($status, "Unsupported extension:") != FALSE) ||
      (strstr($status, "Invalid owner") != FALSE) ||
      (strstr($status, "Unable to write") != FALSE)) {
      fclose($fdeploy);
      return FALSE;
    }
  }
  fclose($fdeploy);
  return "active";
}
/**
 * Undeploys an archive on a PageBox.
 * Makes a request update.php?delete=...
 * Returns true: undeployed or false: PageBox doesn't answer.
 */
function undeploy($loc, $arch) {
  $query = $loc . '?delete=' . $arch;
  // echo $query, "<br>";
  $fdeploy = fopen($query, "r");
  if ($fdeploy == FALSE) {
    return false;
  }
  fclose($fdeploy);
  return true;
}
/**
 * Returns false if already locked and in same process. Hung if different process.
 */
function lock($script, $user, $host) {
  $locked = true;
  if (file_exists("lock.rep")) {
    $fp = fopen("lock.rep", "r+");
    flock($fp, LOCK_EX);
    $lock = fread($fp, 1);
    if ($lock != "1") {
      fwrite($fp, "1");
      $locked = false;
    }
    flock($fp, LOCK_UN);
    fclose($fp);
  } else {
    $fp = fopen("lock.rep", "w");
    if ($fp != FALSE) {	// if unable to write no lock
      flock($fp, LOCK_EX);
      fwrite($fp, "1");
      flock($fp, LOCK_UN);
      fclose($fp);
    }
    $locked = false;
  }
  if ($locked) {
    return false;
  }
  $faudit = fopen("../repository/audit.html", "a");
  if ($faudit != FALSE) {
    fwrite($faudit, "<tr><td>" . date("d M Y h:i:s") . "</td><td>" . $host . "</td><td>" . $user .
      "</td><td>" . $script . "</td></tr>\r\n");
    fclose($faudit);
  }
  return true;
}
/**
 * Unlocks.
 */
function unlock() {
  $fp = fopen("lock.rep", "w");
  if ($fp == FALSE) { // if unable to write no lock
    return;
  }
  flock($fp, LOCK_EX);
  fwrite($fp, "0");
  flock($fp, LOCK_UN);
  fclose($fp);
}
/**
 * Audit
 */
function audit($cmd, $user, $host) {
  $faudit = fopen("../repository/audit.html", "a");
  if ($faudit != FALSE) {
    fwrite($faudit, "<tr><td>" . date("d M Y h:i:s") . "</td><td>" . $host . "</td><td>" . $user .
      "</td><td>" . $cmd . "</td></tr>\r\n");
    fclose($faudit);
  }
}
/**
 * Cleanup.
 * Scans subscribers and tries removing pending states.
 */
function cleanup($download) {
  $RepoSubsObj = restore_subscribers($download);
  $toSave = false;
  $archRep = NULL;
  if ($archRep == NULL) {
    $archRep = restore_archives($this->download);
    if (($archRep == NULL) || ($archRep->archives == NULL)) {
      return "Invalid archive repository (archive.rep)";
    }
  }
  if ($RepoSubsObj->subscribers != NULL) {
    while (list($key, $value) = each ($RepoSubsObj->subscribers)) {
      if ($value->arch == NULL) {
        continue;
      }
      while (list($akey, $avalue) = each ($value->arch)) {
        if ($avalue == "pending deploy") {
          $d = deploy($key, $akey, $archRep->archives[$akey],
            $archRep->docs[$akey], $download);
          if ($d == FALSE) {
            unset($RepoSubsObj->subscribers[$key]->arch[$akey]);
          } else {
            $RepoSubsObj->subscribers[$key]->arch[$akey] = $d;
          }
          $toSave = true;
        } elseif ($avalue == "pending undeploy") {
          if (undeploy($key, $akey)) {
            unset($RepoSubsObj->subscribers[$key]->arch[$akey]);
          }
          $toSave = true;
        }
      }
      if (($value->pbState == "pending unsubscribe") &&
        ($RepoSubsObj->subscribers[$key]->arch == NULL)) {
        unset($RepoSubsObj->subscribers[$key]);
      }
    }
  }
  if ($RepoSubsObj->asubscribers != NULL) {
    while (list($key, $value) = each ($RepoSubsObj->asubscribers)) {
      if ($value->arch == NULL) {
        continue;
      }
      while (list($akey, $avalue) = each ($value->arch)) {
        if ($avalue == "pending deploy") {
          $d = deploy($key, $akey, $archRep->archives[$akey],
            $archRep->docs[$akey], $download);
          if ($d == FALSE) {
            unset($RepoSubsObj->asubscribers[$key]->arch[$akey]);
          } else {
            $RepoSubsObj->asubscribers[$key]->arch[$akey] = $d;
          }
          $toSave = true;
        } elseif ($avalue == "pending undeploy") {
          if (undeploy($key, $akey)) {
            unset($RepoSubsObj->asubscribers[$key]->arch[$akey]);
          }
          $toSave = true;
        }
      }
      if (($value->pbState == "pending unsubscribe") &&
        ($RepoSubsObj->asubscribers[$key]->arch == NULL)) {
        unset($RepoSubsObj->asubscribers[$key]);
      }
    }
  }
  if ($toSave) {
    save_subscribers($RepoSubsObj);
    return "Cleanup performed";
  } else {
    return "No archive in pending state";
  }
}
?>